%binomialator
% GUI to investigate the Binomial Distribution, and perform associated
% calculations. The tendancy of the distribution to become Poisson or
% Normal is also represented.
%
% LAST UPDATED by Andy French January 2016

function varargout = binomialator(varargin)

%% Initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
    'gui_Singleton',  gui_Singleton, ...
    'gui_OpeningFcn', @binomialator_OpeningFcn, ...
    'gui_OutputFcn',  @binomialator_OutputFcn, ...
    'gui_LayoutFcn',  [] , ...
    'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end

%%

%% Harmonograph opening function. This executes just before binomialator is made visible.
function binomialator_OpeningFcn(hObject, eventdata, handles, varargin)

% Choose default command line output for binomialator
handles.output = hObject;

%Turn off warnings (e.g. 'divide by zero')
warning off

%Set up global variable d which can be accessed by all functions
global d
d.h = handles;

%Center figure
figcentre( d.h.FIGUREbinomialator );

%Set defaults
binomialator_defaults

%Update binomialator
binomialator_update;

%%

% Default parameters
function binomialator_defaults
global d

%Define default parameters
d.Nmax = 100;
d.N = 40;
d.p = 0.5;
d.xmin = 0;
d.xmax = 70;
d.ymax = 0.2;
d.x = 10;
d.poisson = 0;
d.normal = 0;
d.plot_and_save_PNG = 0;
d.fsize = 18;

%Set GUI parameters
set( d.h.EDITxmin,'string',num2str(d.xmin) );
set( d.h.EDITxmax,'string',num2str(d.xmax) );
set( d.h.EDITymax,'string',num2str(d.ymax) );
set( d.h.EDITx,'string',num2str(d.x) );
set( d.h.EDITN,'string',num2str(d.N) );
set( d.h.EDITNmax,'string',num2str(d.Nmax) );
set( d.h.EDITp,'string',num2str(d.p) );
set( d.h.SLIDERN,'value',1 - ( d.Nmax - d.N )/d.Nmax );
set( d.h.SLIDERp,'value', 1 - d.p );

%%

%% Diffractor output function. Outputs from this function are returned to the command line.
function varargout = binomialator_OutputFcn(hObject, eventdata, handles)
global d

% Get default command line output from handles structure
varargout{1} = d.h.output;

%%

%% Toggle Poission distribution overlay
function TOGGLEpoisson_Callback(hObject, eventdata, handles)
global d
if d.poisson==1
    d.poisson = 0;
else
    d.poisson = 1;
end
binomialator_update

%%

%% Toggle Normal distribution overlay
function TOGGLEnormal_Callback(hObject, eventdata, handles)
global d
if d.normal==1
    d.normal = 0;
else
    d.normal = 1;
end
binomialator_update

%%

%% Save PNG of graph
function PUSHsavepng_Callback(hObject, eventdata, handles)
global d
d.plot_and_save_PNG = 1;
binomialator_update;
d.plot_and_save_PNG = 0;

%%

%% Number of binary trials slider
function SLIDERN_Callback(hObject, eventdata, handles)
global d

%Compute value of slider
v = get(handles.SLIDERN,'value');

%Determine vale of N and update associated edit box
d.N = 1 + v * ( d.Nmax - 1 );
d.N = round(d.N);
set(d.h.EDITN,'string',num2str(d.N));

%Update GUI
binomialator_update

%%

%% Trial probability slider
function SLIDERp_Callback(hObject, eventdata, handles)
global d

%Compute value of slider
d.p = get(handles.SLIDERp,'value');

%Determine vale of p and update associated edit box
set(d.h.EDITp,'string',num2str(d.p));

%Update GUI
binomialator_update

%%

%% Number of binary trials slider
function EDITN_Callback(hObject, eventdata, handles)
global d

%Update value of d.N
d.N = str2num( get( d.h.EDITN, 'string' ) );

%Clip invalid values
if d.N>d.Nmax
    d.N=d.Nmax;
elseif d.N<1
    d.N = 1;
end
set( d.h.EDITN, 'string', num2str( d.N ) );

%Compute value of slider and update
v = (d.N-1)/(d.Nmax-1);
set( d.h.SLIDERN,'value',v);

%Update GUI
binomialator_update

%%

%% Trial probability slider
function EDITp_Callback(hObject, eventdata, handles)
global d

%Update value of d.p
d.p = str2num( get( d.h.EDITp, 'string' ) );

%Clip invalid values
if d.p>1
    d.p=1;
elseif d.p<0
    d.p = 0;
end
set( d.h.EDITp, 'string', num2str( d.p ) );

%Update slider
set( d.h.SLIDERp,'value',d.p);

%Update GUI
binomialator_update

%%


%x value edit box
function EDITx_Callback(hObject, eventdata, handles)
global d
d.x = str2num( get( d.h.EDITx,'string') );
d.x = round(d.x);
set( d.h.EDITx,'string',num2str(d.x) );
binomialator_update

%%

%xmin value edit box
function EDITxmin_Callback(hObject, eventdata, handles)
global d
d.xmin = str2num( get( d.h.EDITxmin,'string') );
if d.xmin<0
    d.xmin = 0;
end
set( d.h.EDITxmin,'string',num2str(d.xmin) );
binomialator_update

%%

%xmax value edit box
function EDITxmax_Callback(hObject, eventdata, handles)
global d
d.xmax = str2num( get( d.h.EDITxmax,'string') );
if d.xmax<=d.xmin
    %If value set is invalid, use std of Binomial distribution to separate
    %the values of the plot.
    d.xmax = d.xmin + 4*sqrt( d.N*d.p*(1-d.p) );
end
set( d.h.EDITxmax,'string',num2str(d.xmax) );
binomialator_update

%%

%ymax value edit box
function EDITymax_Callback(hObject, eventdata, handles)
global d
d.ymax = str2num( get( d.h.EDITymax,'string') );
if d.ymax<0
    d.ymax = 1;
end
binomialator_update

%%

%Nmax value edit box
function EDITNmax_Callback(hObject, eventdata, handles)
global d
d.Nmax = str2num( get( d.h.EDITNmax,'string') );
if d.Nmax<0
    d.Nmax = 1;
    d.N = 1;
elseif d.Nmax < d.N
    d.N = d.Nmax;
end
set( d.h.EDITN,'string',num2str(d.N));

%Compute value of slider and update
v = (d.N-1)/(d.Nmax-1);
set( d.h.SLIDERN,'value',v);

%Update GUI
binomialator_update

%%

%Center axis scale push button
function PUSHcenter_Callback(hObject, eventdata, handles)
global d
d.xmin = floor( d.N*d.p - 4*sqrt( d.N*d.p*(1-d.p) ) );
d.xmax = ceil( d.N*d.p + 4*sqrt( d.N*d.p*(1-d.p) ) );
set( d.h.EDITxmin, 'string', num2str( d.xmin ) );
set( d.h.EDITxmax, 'string', num2str( d.xmax ) );
binomialator_update

%%

%GUI update function
function binomialator_update
global d

%Save externally plotted graph, if option set. Otherwise update  GUI plot.
if d.plot_and_save_PNG==1
    fig = figure;
    ax = axes;
else
    figure( d.h.FIGUREbinomialator );
    axes( d.h.AXESbinomialator );
    cla;
end

%Plot graph of binomial probabilities
x = ceil( d.xmin ) : floor( d.xmax );
p = B( x, d.N, d.p );
plot( x,p,'ro');
xlim( [d.xmin,d.xmax] );
ylim( [0,d.ymax] );
xlabel('x','fontsize',d.fsize );
ylabel('P(x)','fontsize',d.fsize );
title( ['Binomial Distribution N = ',num2str(d.N),', p = ',num2str(d.p) ], 'fontsize', d.fsize );
grid on

%Overlay poisson distribution if option set
if d.poisson==1
    p_poisson = poisson( x, d.N*d.p );
    hold on;
    plot( x, p_poisson,'g');
end

%Overlay normal distribution if option set
if d.normal==1
    x = linspace(d.xmin,d.xmax,1000);
    p_normal = normal( x, d.N*d.p, sqrt( d.N*d.p*(1-d.p) ) );
    hold on;
    plot( x, p_normal,'b');
end

%Check if x value is valid
if d.x > max(x)
    d.x = max(x);
    set( d.h.EDITx,'string',d.x );
end

%Calculate Binomial probabilities for calculation pane
set( d.h.EDITProb, 'string', num2str( B(d.x,d.N,d.p) ) );
set( d.h.EDITcumProb, 'string', num2str( cumB(d.x,d.N,d.p) ) );
set( d.h.EDITexpectationx,'string',num2str( d.N*d.p ) );
set( d.h.EDITstdx,'string',num2str( sqrt( d.N*d.p*(1-d.p) ) ) );

%Save externally plotted graph, if option set.
if d.plot_and_save_PNG==1
    filename = [ 'binomial N=',num2str(d.N),' p=',num2str(d.p),'.png' ];
    print( fig, filename, '-dpng','-r300' );
    close(fig);
end

%%

%figcenter
% Centres a figure with handle fig in the current display.

function figcentre(fig)

%Get screen dimensions in pixels.
set(0,'units','pixels') ;
screen_size=get(0,'Screensize');
display_size=get(0,'MonitorPositions');

%Get original figure units
orig_units = get(fig,'units');

%Get position vector for figure in pixels
set(fig,'units','pixels');
fig_position=get(gcf,'position') ;

%Define coordinates or figure bottom hand corner from screen bottom hand corner
%that will place the figure window in the center of the screen.
centre_x = 0.5*( display_size(3)-fig_position(3) ) - screen_size(1);
centre_y = 0.5*( display_size(4)-fig_position(4) ) - screen_size(2);

%Modify figure position to centre
set(fig,'position',[centre_x,centre_y,fig_position(3),fig_position(4)] );

%Reset figure units
set(fig,'units',orig_units);

%%

%Binomial distribution function
function P = B(x,N,p)
P = zeros(size(x));
for n=1:length(x)
    if ( x(n) <= N ) && ( x(n) >= 0 )
        P(n) = nchoosek(N,x(n))*(p^x(n)).*(1-p).^(N-x(n)) ;
    else
        P(n) = 0;
    end
end

%%

%Cumulative Binomial distrbution function
function P = cumB(x,N,p)
for n = 1:length(x)
    xx = ceil(x(n));
    PP = 0;
    for k=0:xx
        PP = PP + B(k,N,p);
    end
    P(n) = PP;
end

%%

%Poisson distribution function
function P = poisson(x,lamda)
x = round(x);
invalid_x = x<0;
x(invalid_x) = 0;
P = ((lamda.^x)./(factorial(x)))*exp(-lamda);
P(invalid_x) = 0;

%%

%Normal distribution function
function P = normal( x, mu, sigma )
P = (1/(sigma*sqrt(2*pi)))*exp( -((x-mu).^2)/(2*sigma^2) );

%End of code






